require(['jquery'], function ($) {
  function filterTable(event) {
            console.log("Filter")

    var input = $(event.target);
    var index = input.closest("th").index();
    var table = input.closest("table");
    var query = (input.val() || "").toLowerCase();
    table.find("tbody > tr").each(function(_, tr) {
      // Do not filter header -- todo: th should be inside thead
      var td = $(tr).children("td").eq(index);
      if (td.length === 1) {
        var text = (td.text() || "").toLowerCase();
        if (query.length === 0 || text.indexOf(query) >= 0) {
          $(tr).removeClass("hidden");
        }
        else {
          $(tr).addClass("hidden");
        }
      }
    });
  }

  var getTrList = function(table,includeTH) {
    var tr = table.find("tr").first()
    var trList = tr.siblings().addBack(tr)
    if (!includeTH) {
      trList = trList.filter(function(){return $(this).children("th").length===0})
    }
    return trList
  }
  var resetTH = function(thList) {
    thList.find(".xm-sort-icon").removeClass("fa-sort-numeric-asc fa-sort-numeric-desc fa-sort-alpha-asc fa-sort-alpha-desc").addClass("fa-sort")
  }
  var triggerIcons = function(thList,span,type,dir) {
    if (type==='a') {
      resetTH(thList)
      if (dir === 1) {
        span.removeClass("fa-sort-alpha-desc")
        span.addClass("fa-sort-alpha-asc")
      }
      else if (dir === -1) {
        span.removeClass("fa-sort-alpha-asc fa-sort")
        span.addClass("fa-sort-alpha-desc")
      }
    }
    else if (type === 'n') {
      resetTH(thList)
      if (dir === 1) {
        span.removeClass("fa-sort-numeric-desc fa-sort")
        span.addClass("fa-sort-numeric-asc")
      }
      else if (dir === -1) {
        span.removeClass("fa-sort-numeric-asc fa-sort")
        span.addClass("fa-sort-numeric-desc")
      }
    }
    else if (type === 'd') {
      resetTH(thList)
      if (dir === 1) {
        span.removeClass("fa-sort-amount-desc fa-sort")
        span.addClass("fa-sort-amount-asc")
      }
      else if (dir === -1) {
        span.removeClass("fa-sort-amount-asc fa-sort")
        span.addClass("fa-sort-amount-desc")
      }
    }
  }
  var sort = function(table,idx,dir) {
    var trList = getTrList(table,true)
    var parent = trList.first().parent()
    var thList = trList.children("th") 
    var span = $(thList.get(idx)).find(".xm-sort-icon")
    var type = table.data("tableHash")[idx]
    var dateFormat = table.data("dateFormat") || null
    trList = trList.filter(function(){return $(this).children("th").length===0})
    if (type === undefined || type === "") return
    // Get sort keys.
    trList.each(function(){
      var tr = $(this)
      var td = $(tr.children("td").get(idx))
      var text = String(td.text())
      if (type === "a") {
        if (text.toLocaleLowerCase) text = text.toLocaleLowerCase()
        else if (text.toLowerCase) text = text.toLowerCase()
        if (text.trim) text = text.trim()
        tr.data("sortKey",text)
      }
      else if (type === "n") {
        var num = parseFloat(text.replace(/[^0-9,.]/g,"").replace(',','.')) || 0
        tr.data("sortKey",num)
      }
      else if (type === "d") {
        var date
        if (dateFormat) date = $.jdate.strptime(text,dateFormat)
        if (date) date = date.getTime()
        else date = Date.parse(text)
        if (!date) date = 0
        tr.data("sortKey",date)
      }
    })
    // Sort by the key.
    trList.detach()
    triggerIcons(thList,span,type,dir)
    trList.sort(function(x,y){
      var a = $(x).data("sortKey")
      var b = $(y).data("sortKey")
      return dir * (a<b ? -1 : a>b ? 1 : 0)
    })
    parent.append(trList)
  }

  var adjustWidth = function(table) {
    var trList
    var w = table.attr("data-xm-table-width")
    if (w) table.children("table").css("width",w)
    var colw = table.attr("data-xm-table-cw")
    colw = colw ? colw.split('-') : []
    if (colw.length>0) trList = getTrList(table)
    for (var i=0; i<colw.length; ++i) {
      w = colw[i]
      if (w) {
        trList.each(function(){
          $($(this).children("td,th").get(i)).css({"max-width":w + "px","overflow-x":"hidden"})
        })
      }
    }
  }

  $(".xm-table").each(function(){
    var me = $(this)
    var typeHash = {}
    var dta = me.attr("data-xm-table-dta")
    var dtn = me.attr("data-xm-table-dtn")
    var dtd = me.attr("data-xm-table-dtd")
    var ps = me.attr("data-xm-table-ps")
    var cap = me.attr("data-xm-table-cap")
    var fcol = me.attr("data-xm-table-fcol")
    var dateFormat = me.attr("data-xm-df")
    dta = dta ? dta.split("-") : []
    fcol = fcol ? fcol.split("-") : []
    dtn = dtn ? dtn.split("-") : []
    dtd = dtd ? dtd.split("-") : []
    dta.forEach(function(x){typeHash[parseInt(x)] = "a"})
    dtn.forEach(function(x){typeHash[parseInt(x)] = "n"})
    dtd.forEach(function(x){typeHash[parseInt(x)] = "d"})
    if (dateFormat) me.data("dateFormat", String(dateFormat))
    var trList = getTrList(me,true)
    var thList = trList.children("th")
    me.data("tableHash",typeHash)
    // add caption
    if (cap && cap.length>0) {
      var caption = $("<caption>")
      caption.text(cap)
             .addClass("xm-table-caption")
      me.find("table").first().prepend(caption)
    }
    adjustWidth(me)
    if (thList.length > 0 && trList.length > 2) {
      thList.each(function(idx){
        // Filtering
        if (fcol.indexOf(String(idx)) >= 0) {
          var input = $("<input>", {
            class: "xm-table-filter",
            placeholder: "search...",
            type: "text",
          });
          thList.eq(idx).append(input);
          input.on("keyup change", filterTable);
        }

        // Sorting
        var type = typeHash[idx]
        if (type !== undefined && type !== "") {
          var th = $(this)
          var span = $("<span>")
          span.addClass("fa fa-sort xm-sort-icon xm-sort-" + type)
          th.addClass("xm-column-sortable")
          th.append(span)
          th.data("col",idx)
          // Sort the table.
          th.on("click",function(){
            var table = $(this).closest(".xm-table")
            var dir
            var idx = $(this).data("col")
            if (span.hasClass("fa-sort-alpha-desc")) dir = 1
            else if (span.hasClass("fa-sort-alpha-asc")) dir = -1
            else if (span.hasClass("fa-sort-numeric-desc")) dir = 1
            else if (span.hasClass("fa-sort-numeric-asc")) dir = -1
            else if (span.hasClass("fa-sort-amount-desc")) dir = 1
            else if (span.hasClass("fa-sort-amount-asc")) dir = -1
            else if (span.hasClass("xm-sort-a") || span.hasClass("xm-sort-n") || span.hasClass("xm-sort-d")) dir = 1
            if (dir) sort(table,idx,dir)
          })
        }
      })
    }
    if (ps) {
      var fields = ps.split("-")
      psCol = parseInt(ps[0]) || 0
      psDir = ps[1] === 'asc' ? 1 : ps[4] === 'desc' ? -1 : '1'
      sort(me,psCol,psDir)
    }
  })
})
